//
//  NativeScannerView.swift
//  Runner
//
//  Created by Sergio Durban Belmonte on 30/11/21.
//

import Foundation
import AVFoundation

public protocol NativeScannerViewDelegate: NSObjectProtocol {
    func barcodeOutput(string: String?)
}

final class NativeScannerView: UIView, AVCaptureVideoDataOutputSampleBufferDelegate {
    private let videoPreviewLayer = AVCaptureVideoPreviewLayer()
    private let captureSession = AVCaptureSession()
    
    // Detectors
    internal let visionBarcodeDetect = VisionBarcodeDetector()
    
    // Delegate
    public var delegate: NativeScannerViewDelegate? = nil
    
    public func startReader() {
        guard let device = AVCaptureDevice.default(for: .video) else {
            return
        }
        
        captureSession.beginConfiguration()
        
        let videoOutput = AVCaptureVideoDataOutput()
        videoOutput.alwaysDiscardsLateVideoFrames = true
        
        guard let deviceInput = try? AVCaptureDeviceInput(device: device),
              captureSession.canAddInput(deviceInput),
              captureSession.canAddOutput(videoOutput) else {
                  return
              }
        
        do {
            try device.lockForConfiguration()
        } catch {
            return
        }
        
        /// Se ejecutara al final del init lo colocamos aqui para acordarnos de quitar el unlock y de commitear la config
        defer {
            device.unlockForConfiguration()
            captureSession.commitConfiguration()
        }
        
        device.isSubjectAreaChangeMonitoringEnabled = true
        captureSession.addInput(deviceInput)
        captureSession.addOutput(videoOutput)
        
            /// Aqui podemos aumentar la calidad de la imagen capturada en vivo ya que es el preset que vamos a aplicar a la camara.
        let photoPreset = AVCaptureSession.Preset.photo
        
        if captureSession.canSetSessionPreset(photoPreset) {
            captureSession.sessionPreset = photoPreset
        }
        
        videoOutput.setSampleBufferDelegate(self, queue: DispatchQueue(label: "video_ouput_queue"))
        
        videoPreviewLayer.session = captureSession
        videoPreviewLayer.backgroundColor = UIColor(displayP3Red: 0, green: 0, blue: 0, alpha: 1).cgColor
        videoPreviewLayer.connection?.videoOrientation = .portrait
        videoPreviewLayer.videoGravity = .resizeAspectFill
    }
    
    // MARK: - When UVIew shows
    override func willMove(toWindow newWindow: UIWindow?) {
        if newWindow != nil {
            self.videoPreviewLayer.frame = layer.bounds
            
            self.layer.addSublayer(self.videoPreviewLayer)
            
            if !hasPermissions() {
                requestVideoPermissions { completion in
                    if completion && self.hasPermissions() {
                        self.captureSession.startRunning()
                    }
                }
            } else {
                captureSession.startRunning()
            }
        } else {
            captureSession.stopRunning()
        }
    }
    
    
    // MARK: - Video Permissions
    public func hasPermissions() -> Bool {
        let authorizationStatus = AVCaptureDevice.authorizationStatus(for: .video)
        
        switch authorizationStatus {
            case .authorized:
                return true
            case .notDetermined:
                return false
            default:
                return false
        }
    }
    
    public func requestVideoPermissions(completion: @escaping (_ response: Bool) -> Void) {
        AVCaptureDevice.requestAccess(for: .video, completionHandler: { response in
            completion(response)
        })
    }
    
    // MARK: - AVCaptureVideoDataOutputSampleBufferDelegate
    func captureOutput(_ output: AVCaptureOutput, didOutput sampleBuffer: CMSampleBuffer, from connection: AVCaptureConnection) {
        guard let pixelBuffer = CMSampleBufferGetImageBuffer(sampleBuffer) else {
            return
        }
        
        self.visionBarcodeDetect.barcode(forPixelBuffer: pixelBuffer) { barcode in
            delegate?.barcodeOutput(string: barcode)
        }
    }
    
}
