import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';
import 'package:injectable/injectable.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:scanfood/application/scan_result_bloc/scan_result_cubit.dart';
import 'package:scanfood/domain/model/value_objects/barcode.dart';
import 'package:scanfood/infrastructure/native/controller/native_scanner_controller.dart';

part 'camera_event.dart';
part 'camera_state.dart';

@injectable
class CameraBloc extends Bloc<CameraEvent, CameraState> {
  final ScanResultCubit _scanResultCubit;
  late NativeScannerController _nativeScannerController;
  StreamSubscription? barcodeStreamSubscription;
  Set<Barcode> barcodes = {};
  Set<Barcode> analyzedBarcodes = {};

  CameraBloc(this._scanResultCubit) : super(CameraInitial()) {
    _restartCameraController();

    // Listeners
    on<CameraReady>(_viewReady);
    on<_CameraBarcodeFound>(_barcodeFound);
    on<CameraClear>(_clearCamera);
    on<ReloadCamera>(_reloadCamera);

    _scanResultCubit.clearProducts();
    barcodes.clear();
    analyzedBarcodes.clear();
  }

  void _restartCameraController() {
    _nativeScannerController = NativeScannerController(callbackListen: () {
      barcodeStreamSubscription = _nativeScannerController.barcodeStream.listen((event) {
        add(_CameraBarcodeFound(barcode: Barcode(code: event)));
      });
    });
  }

  @override
  Future<void> close() async {
    await barcodeStreamSubscription?.cancel();
    return super.close();
  }

  Future<void> _viewReady(
    CameraReady event,
    Emitter<CameraState> emit,
  ) async {
    await Permission.camera.request();

    if (await Permission.camera.status != PermissionStatus.granted) {
      emit(const CameraHasNoPermissionsEvent());
      _restartCameraController();
    } else {
      emit(CameraReadyEvent(
        eventChannelName: _nativeScannerController.eventChannelName,
        methodChannelName: _nativeScannerController.channelName,
        scannedElements: analyzedBarcodes.length,
      ));
    }
  }

  Future<void> _barcodeFound(
    _CameraBarcodeFound event,
    Emitter<CameraState> emit,
  ) async {
    final int currentLength = barcodes.length;
    barcodes.add(event.barcode);

    if (currentLength < barcodes.length) {
      emit(CameraScannedCodeLoading(scannedElements: analyzedBarcodes.length));

      try {
        await _scanResultCubit.addBarcode(
          event.barcode,
        );
        analyzedBarcodes.add(event.barcode);
      } catch (_) {
        return emit(CameraScannedCodeError(scannedElements: analyzedBarcodes.length));
      }
    }

    emit(CameraScannedCode(scannedElements: analyzedBarcodes.length));
  }

  void _clearCamera(
    CameraClear event,
    Emitter<CameraState> emit,
  ) {
    barcodes.clear();
    analyzedBarcodes.clear();

    emit(CameraScannedCode(scannedElements: analyzedBarcodes.length));
  }

  Future<void> _reloadCamera(
    ReloadCamera event,
    Emitter<CameraState> emit,
  ) async {
    _restartCameraController();
    add(CameraReady());
  }
}
