import 'package:collection/collection.dart';
import 'package:openfoodfacts/model/NutrientLevels.dart';
import 'package:openfoodfacts/model/Product.dart';
import 'package:scanfood/domain/model/objects/product.dart' as DomainProduct;
import 'package:scanfood/domain/model/value_objects/barcode.dart';

extension OpenFoodFactsProductToProduct on Product {
  DomainProduct.Product toProduct() {
    return DomainProduct.Product(
      productName: productName ?? '',
      brand: brands ?? '',
      quantity: quantity ?? '',
      barcode: barcode != null ? Barcode(code: barcode!) : const Barcode(code: ''),
      sugarsLevel: _levelToNutrientLevel(nutrientLevels?.levels[NutrientLevels.NUTRIENT_SUGARS] ?? Level.UNDEFINED),
      fatLevel: _levelToNutrientLevel(nutrientLevels?.levels[NutrientLevels.NUTRIENT_FAT] ?? Level.UNDEFINED),
      saturatedFatLevel:
          _levelToNutrientLevel(nutrientLevels?.levels[NutrientLevels.NUTRIENT_SATURATED_FAT] ?? Level.UNDEFINED),
      saltLevel: _levelToNutrientLevel(nutrientLevels?.levels[NutrientLevels.NUTRIENT_SALT] ?? Level.UNDEFINED),
      ecoScoreLevel: _ecoScoreGradeToEcoScoreLevel(ecoscoreGrade),
      nutriScoreLevel: _nutriScoreGradeToNutriScoreLevel(nutriscore),
      nova: _novaAttributeToNovaLevel(attributeGroups
          ?.toList()
          .firstWhereOrNull((element) => element.id == 'processing')
          ?.attributes
          ?.toList()
          .firstWhereOrNull((element) => element.id == 'nova')
          ?.title),
      allergens:
          allergens?.ids.map(_allergensApiToAllegens).whereType<DomainProduct.Allergens>().toSet().toList() ?? [],
      imageUrl: imageFrontUrl ?? '',
      fat: nutriments?.fat,
      salt: nutriments?.salt,
      saturatedFat: nutriments?.saturatedFat,
      sugars: nutriments?.sugars,
    );
  }

  DomainProduct.NutrientLevel _levelToNutrientLevel(Level level) {
    DomainProduct.NutrientLevel nutrientLevel;
    switch (level) {
      case Level.LOW:
        nutrientLevel = DomainProduct.NutrientLevel.low;
        break;
      case Level.MODERATE:
        nutrientLevel = DomainProduct.NutrientLevel.medium;
        break;
      case Level.HIGH:
        nutrientLevel = DomainProduct.NutrientLevel.high;
        break;
      case Level.UNDEFINED:
        nutrientLevel = DomainProduct.NutrientLevel.unknown;
        break;
    }

    return nutrientLevel;
  }

  DomainProduct.EcoScoreLevel _ecoScoreGradeToEcoScoreLevel(String? grade) {
    late DomainProduct.EcoScoreLevel ecoScoreLevel;

    switch (grade) {
      case 'a':
        ecoScoreLevel = DomainProduct.EcoScoreLevel.A;
        break;
      case 'b':
        ecoScoreLevel = DomainProduct.EcoScoreLevel.B;
        break;
      case 'c':
        ecoScoreLevel = DomainProduct.EcoScoreLevel.C;
        break;
      case 'd':
        ecoScoreLevel = DomainProduct.EcoScoreLevel.D;
        break;
      case 'e':
        ecoScoreLevel = DomainProduct.EcoScoreLevel.E;
        break;
      default:
        ecoScoreLevel = DomainProduct.EcoScoreLevel.unknown;
    }

    return ecoScoreLevel;
  }

  DomainProduct.NutriScoreLevel _nutriScoreGradeToNutriScoreLevel(String? grade) {
    late DomainProduct.NutriScoreLevel nutriScoreLevel;

    switch (grade) {
      case 'a':
        nutriScoreLevel = DomainProduct.NutriScoreLevel.A;
        break;
      case 'b':
        nutriScoreLevel = DomainProduct.NutriScoreLevel.B;
        break;
      case 'c':
        nutriScoreLevel = DomainProduct.NutriScoreLevel.C;
        break;
      case 'd':
        nutriScoreLevel = DomainProduct.NutriScoreLevel.D;
        break;
      case 'e':
        nutriScoreLevel = DomainProduct.NutriScoreLevel.E;
        break;
      default:
        nutriScoreLevel = DomainProduct.NutriScoreLevel.unknown;
    }

    return nutriScoreLevel;
  }

  DomainProduct.NOVALevel _novaAttributeToNovaLevel(String? novaAttribute) {
    late DomainProduct.NOVALevel novaLevel;

    switch (novaAttribute) {
      case 'NOVA 1':
        novaLevel = DomainProduct.NOVALevel.A;
        break;
      case 'NOVA 2':
        novaLevel = DomainProduct.NOVALevel.B;
        break;
      case 'NOVA 3':
        novaLevel = DomainProduct.NOVALevel.C;
        break;
      case 'NOVA 4':
        novaLevel = DomainProduct.NOVALevel.D;
        break;
      default:
        novaLevel = DomainProduct.NOVALevel.unknown;
    }

    return novaLevel;
  }

  DomainProduct.Allergens? _allergensApiToAllegens(String allergen) {
    DomainProduct.Allergens? allergens;

    switch (allergen) {
      case 'en:lupin':
        allergens = DomainProduct.Allergens.lupin;
        break;
      case 'en:celery':
        allergens = DomainProduct.Allergens.celery;
        break;
      case 'en:peanuts':
        allergens = DomainProduct.Allergens.peanuts;
        break;
      case 'en:crustaceans':
        allergens = DomainProduct.Allergens.crustaceans;
        break;
      case 'en:sulphur-dioxide-and-sulphites':
        allergens = DomainProduct.Allergens.sulphur_dioxide_and_sulphites;
        break;
      case 'en:nuts':
        allergens = DomainProduct.Allergens.nuts;
        break;
      case 'en:gluten':
        allergens = DomainProduct.Allergens.gluten;
        break;
      case 'en:sesame-seeds':
        allergens = DomainProduct.Allergens.sesame_seeds;
        break;
      case 'en:eggs':
        allergens = DomainProduct.Allergens.eggs;
        break;
      case 'en:milk':
        allergens = DomainProduct.Allergens.milk;
        break;
      case 'en:molluscs':
        allergens = DomainProduct.Allergens.molluscs;
        break;
      case 'en:fish':
        allergens = DomainProduct.Allergens.fish;
        break;
      case 'en:soybeans':
        allergens = DomainProduct.Allergens.soybeans;
        break;
      default:
        allergens = null;
        break;
    }

    return allergens;
  }
}
