import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:scanfood/application/camera_bloc/camera_bloc.dart';
import 'package:scanfood/infrastructure/native/widgets/native_scanner_view.dart';
import 'package:scanfood/resources/l10n/l10n.dart';
import 'package:scanfood/ui/common/common_action_modal.dart';
import 'package:scanfood/ui/common/common_button.dart';
import 'package:scanfood/ui/results_page/results_page.dart';
import 'package:scanfood/ui/theme/app_theme.dart';
import 'package:top_snackbar_flutter/custom_snack_bar.dart';
import 'package:top_snackbar_flutter/top_snack_bar.dart';

part 'widgets/camera_counter.dart';
part 'widgets/camera_rectangle_painter.dart';

class CameraPage extends StatefulWidget {
  final bool pageIsBeingShown;

  CameraPage({
    Key? key,
    required this.pageIsBeingShown,
  }) : super(key: key);

  @override
  State<CameraPage> createState() => _CameraPageState();
}

class _CameraPageState extends State<CameraPage> with WidgetsBindingObserver {
  final CameraBloc _cameraBloc = GetIt.I.get<CameraBloc>();
  bool sendedToSettings = false;

  @override
  void initState() {
    _cameraBloc.add(CameraReady());
    WidgetsBinding.instance!.addObserver(this);
    super.initState();
  }

  @override
  void dispose() {
    WidgetsBinding.instance!.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    switch (state) {
      case AppLifecycleState.resumed:
        if (sendedToSettings) {
          sendedToSettings = false;
          _cameraBloc.add(CameraReady());
        }
        break;
      case AppLifecycleState.inactive:
        break;
      case AppLifecycleState.paused:
        break;
      case AppLifecycleState.detached:
        break;
    }
    super.didChangeAppLifecycleState(state);
  }

  @override
  void activate() {
    print('returned');
    super.activate();
  }

  @override
  Widget build(BuildContext context) {
    if (!widget.pageIsBeingShown) {
      return const SizedBox.shrink();
    }

    return Scaffold(
      body: Stack(
        children: [
          BlocBuilder(
            bloc: _cameraBloc,
            buildWhen: (previous, current) => current is CameraScannedCodeError,
            builder: (context, CameraState state) {
              if (state is CameraScannedCodeError && widget.pageIsBeingShown) {
                WidgetsBinding.instance!.addPostFrameCallback((_) {
                  showTopSnackBar(
                    context,
                    CustomSnackBar.error(
                      message: S.of(context).error_reading_barcode,
                    ),
                  );
                });
              }

              return Container();
            },
          ),
          BlocBuilder(
            bloc: _cameraBloc,
            buildWhen: (previous, current) => current is CameraHasNoPermissionsEvent,
            builder: (context, CameraState state) {
              if (state is CameraHasNoPermissionsEvent && widget.pageIsBeingShown) {
                WidgetsBinding.instance?.addPostFrameCallback((_) {
                  showDialog(
                    context: context,
                    barrierColor: AppTheme.neutral900.withOpacity(0.9),
                    barrierDismissible: true,
                    builder: (_) {
                      return CommonActionModal(
                        title: S.of(context).camera_permission_denied_title,
                        subtitle: S.of(context).camera_permission_denied_description,
                        horizontalButtons: [
                          CommonButton.settingsButton(
                            onPressed: () async {
                              await openAppSettings();
                              Navigator.of(context).pop();
                              sendedToSettings = true;
                            },
                            title: S.of(context).camera_permissions_denied_go_settings,
                          ),
                        ],
                      );
                    },
                  );
                });
              }

              return Container(
                color: Colors.black,
              );
            },
          ),
          Positioned.fill(
              child: BlocBuilder(
            bloc: _cameraBloc,
            buildWhen: (previous, current) => current is CameraReadyEvent,
            builder: (context, state) {
              Widget widget = const SizedBox.shrink();

              if (state is CameraReadyEvent) {
                widget = _CameraScanner(
                  methodChannelId: state.methodChannelName,
                  eventChannelName: state.eventChannelName,
                );
              }

              return widget;
            },
          )),
          BlocBuilder(
            bloc: _cameraBloc,
            buildWhen: (previous, current) =>
                current is CameraReadyEvent || current is CameraScannedCode || current is CameraScannedCodeLoading,
            builder: (context, CameraState state) {
              return AnimatedPositionedDirectional(
                child: GestureDetector(
                  onTap: () {
                    Navigator.of(context).pushNamed(ResultPage.ROUTE).then(
                      (_) {
                        _cameraBloc.add(
                          const ReloadCamera(),
                        );
                      },
                    );
                  },
                  child: _CameraCounter(
                    itemsScanned: state.scannedElements,
                    loading: state is CameraScannedCodeLoading,
                  ),
                ),
                end: state.scannedElements > 0 || state is CameraScannedCodeLoading ? 0 : -_CameraCounter.BOX_SIZE,
                top: MediaQuery.of(context).size.height * 0.75,
                duration: const Duration(milliseconds: 300),
              );
            },
          ),
        ],
      ),
    );
  }
}

class _CameraScanner extends StatelessWidget {
  final String methodChannelId;
  final String eventChannelName;

  const _CameraScanner({
    Key? key,
    required this.methodChannelId,
    required this.eventChannelName,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return CustomPaint(
      foregroundPainter: _CameraRectanglePainter(
        colorLine: AppTheme.complementary600,
        topMargin: MediaQuery.of(context).padding.top,
      ),
      child: NativeScannerView(
        methodChannelId: methodChannelId,
        eventChannelId: eventChannelName,
      ),
    );
  }
}
