import 'package:faker/faker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:scanfood/domain/errors/no_product_found_error.dart';
import 'package:scanfood/domain/model/objects/product.dart';
import 'package:scanfood/domain/model/value_objects/barcode.dart';
import 'package:scanfood/domain/repository/cached_product_repository.dart';
import 'package:scanfood/infrastructure/repository/hive/hive_product_repository.dart';
import 'package:scanfood/infrastructure/repository/open_food_facts_product_repository.dart';

import '../model/object/product_mock.dart';
import '../model/value_objects/barcode_mock.dart';
import 'cached_product_repository_test.mocks.dart';

@GenerateMocks([
  HiveProductRepository,
  OpenFoodFactsProductRepository,
])
void main() {
  group('CachedProductRepository', () {
    test('-> findProductByBarcode, not in cache', () async {
      final MockHiveProductRepository mockHiveProductRepository = MockHiveProductRepository();
      final MockOpenFoodFactsProductRepository mockOpenFoodFactsProductRepository =
          MockOpenFoodFactsProductRepository();

      final Barcode barcode = generateBarcodeMock();
      final Product product = generateMockProduct(barcode: barcode);

      when(mockHiveProductRepository.findProductByBarcode(barcode))
          .thenAnswer((realInvocation) => Future.error(NoProductFoundError()));
      when(mockOpenFoodFactsProductRepository.findProductByBarcode(barcode))
          .thenAnswer((realInvocation) => Future.value(product));

      final CachedProductRepository cachedProductRepository = CachedProductRepository(
        mockHiveProductRepository,
        mockOpenFoodFactsProductRepository,
      );

      await cachedProductRepository.findProductByBarcode(barcode);

      verify(mockHiveProductRepository.findProductByBarcode(barcode)).called(1);
      verify(mockOpenFoodFactsProductRepository.findProductByBarcode(barcode)).called(1);
      verify(mockHiveProductRepository.saveProduct(product)).called(1);
    });

    test('-> findProductByBarcode, cached', () async {
      final MockHiveProductRepository mockHiveProductRepository = MockHiveProductRepository();
      final MockOpenFoodFactsProductRepository mockOpenFoodFactsProductRepository =
          MockOpenFoodFactsProductRepository();

      final Barcode barcode = generateBarcodeMock();
      final Product product = generateMockProduct(barcode: barcode);

      when(mockHiveProductRepository.findProductByBarcode(barcode))
          .thenAnswer((realInvocation) => Future.value(product));

      final CachedProductRepository cachedProductRepository = CachedProductRepository(
        mockHiveProductRepository,
        mockOpenFoodFactsProductRepository,
      );

      await cachedProductRepository.findProductByBarcode(barcode);

      verify(mockHiveProductRepository.findProductByBarcode(barcode)).called(1);
      verifyNever(mockOpenFoodFactsProductRepository.findProductByBarcode(barcode));
      verifyNever(mockHiveProductRepository.saveProduct(product));
    });

    test('-> getAll', () async {
      final MockHiveProductRepository mockHiveProductRepository = MockHiveProductRepository();
      final MockOpenFoodFactsProductRepository mockOpenFoodFactsProductRepository =
          MockOpenFoodFactsProductRepository();

      final Faker faker = Faker();
      final int items = faker.randomGenerator.integer(10, min: 1);
      final List<Product> p = [];

      for (var i = 0; i < items; i++) {
        p.add(generateMockProduct());
      }

      when(mockHiveProductRepository.getAll()).thenAnswer((realInvocation) => Future.value(p));

      final CachedProductRepository cachedProductRepository = CachedProductRepository(
        mockHiveProductRepository,
        mockOpenFoodFactsProductRepository,
      );

      final products = await cachedProductRepository.getAll();

      verify(mockHiveProductRepository.getAll()).called(1);
      expect(products, equals(p));
    });

    test('-> searchByText', () async {
      final MockHiveProductRepository mockHiveProductRepository = MockHiveProductRepository();
      final MockOpenFoodFactsProductRepository mockOpenFoodFactsProductRepository =
          MockOpenFoodFactsProductRepository();

      final Faker faker = Faker();
      final int items = faker.randomGenerator.integer(10, min: 1);
      final List<Product> p = [];

      for (var i = 0; i < items; i++) {
        p.add(generateMockProduct());
      }

      final String rndString = faker.randomGenerator.string(15);

      when(mockOpenFoodFactsProductRepository.searchByText(rndString)).thenAnswer((realInvocation) => Future.value(p));

      final CachedProductRepository cachedProductRepository = CachedProductRepository(
        mockHiveProductRepository,
        mockOpenFoodFactsProductRepository,
      );

      final products = await cachedProductRepository.searchByText(rndString);

      verify(mockOpenFoodFactsProductRepository.searchByText(rndString)).called(1);
      expect(products, equals(p));
    });
  });
}
